#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <termios.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAIN_C
#include "exitfail.h"

#define SERIAL_BAUDRATE B115200

#define BUF_SIZE 2048

static int serial_fd = -1;
static struct termios old_tio;

static int terminated = 0;

#define BASENAME(p) ((strrchr((p), '/') ? : ((p) - 1)) +1)
#define ARRAY_SIZE(a) (sizeof(a) /sizeof((a)[0]))

#define __unused __attribute__((unused))

static void terminate_sig_handler(__unused int signum)
{
	terminated = 1;
}

static void set_sig_handler(int sig_list[], ssize_t num, __sighandler_t handler)
{
	struct sigaction sa;
	int i;

	memset(&sa, 0, sizeof(sa));
	sa.sa_handler = handler;

	for(i=0; i<num; i++)
		if(sigaction(sig_list[i], &sa, NULL) < 0)
			exitfail_errno("sigaction");
}

static void restore_serial(void)
{
	int ret;

	ret = tcsetattr(serial_fd, TCSANOW, &old_tio);
	if(ret < 0) exitfail_errno("tcsetattr");
}

static void setup_serial(int fd)
{
	struct termios tio;
	int ret;

	ret = tcgetattr(fd, &old_tio);
	if(ret) exitfail_errno("tcsgetattr");

	if(atexit(restore_serial)) exitfail_errno("atexit");

	memset(&tio, 0, sizeof(tio));
	tio.c_iflag = IGNBRK | IGNPAR;
	tio.c_cflag = CS8 | CLOCAL | CREAD;
	tio.c_cc[VTIME] = 0;
	tio.c_cc[VMIN] = 1;
	ret = cfsetspeed(&tio, SERIAL_BAUDRATE);
	if(ret < 0) exitfail_errno("cfsetspeed");

	ret = tcflush(fd, TCIFLUSH);
	if(ret < 0) exitfail_errno("tcflush");

	ret = tcsetattr(fd, TCSANOW, &tio);
	if(ret) exitfail_errno("tcsetattr");
}

int main(int argc, char *argv[])
{
	int terminate_sig_list[] = {
		SIGHUP, SIGINT, SIGQUIT, SIGPIPE, SIGTERM
	};

/**** sample: variable: begin ****/
	ssize_t ret;
       char buf[BUF_SIZE];
       char buf1[BUF_SIZE];
	int i;
	FILE *fp;
	int cnt_char;
	ssize_t analen;
/**** sample: variable: end ****/

/**** sample: write data: begin ****/
	char w_data[BUF_SIZE];
	ssize_t w_length;
/**** sample: write data: end ****/

	exitfail_init();

	if(argc < 3) {
		printf("Usage: %s <device> <write_data>\n", BASENAME(argv[0]));
		return EXIT_SUCCESS;
	}

	serial_fd = open(argv[1], O_RDWR | O_NOCTTY);
	if(serial_fd < 0) exitfail_errno("open");

	set_sig_handler(terminate_sig_list, ARRAY_SIZE(terminate_sig_list),
			terminate_sig_handler);

	setup_serial(serial_fd);

/**** sample: write data: begin ****/
	w_length = strlen(argv[2]);
	for(i = 0; i < BUF_SIZE; i++) w_data[i] = '\0';
	strncpy(w_data, argv[2], w_length);
	w_data[w_length] = '\r';
	w_data[w_length + 1] = '\n';
/**** sample: write data: end ****/

/**** sample: Write data and Read response: begin ****/
	ret = write(serial_fd, w_data, strlen(w_data));
	if (ret < 0) exitfail_errno("write");
	printf("\n****************************************************\n");
	printf("-------- Write: %s", w_data);

/**** sample: Read Serial Data:: begin ****/
	cnt_char = 0;
	analen = 0;

	while(!terminated) {
		ret = read(serial_fd, buf, BUF_SIZE);
		if(ret < 0) {
			if(errno == EINTR) continue;
			exitfail_errno("read");
		}
		for(i = 0; i < ret; i++) {
//			printf("%c", buf[i]);
			if(cnt_char < BUF_SIZE - 10) {
				cnt_char++;
				if(buf[i] == '\r') buf1[analen++] = '\n';
				else buf1[analen++] = buf[i];
			}
			else {
				cnt_char = 0;
				analen = 0;
			}

			if(buf[i] == '\n' || buf[i] == '\r') {
				buf1[analen] = '\0';
				printf("%s", buf1);

				fp = fopen("/root/log_serial.txt", "w");
				fprintf(fp, "%s\n", buf1);
				fclose(fp);

/**** sample: execute "sample_action.sh" if it's exist: begin ****/
				if((fp = fopen("/root/sample_action.sh", "r")) != NULL) {
					system("/root/sample_action.sh");
					fclose(fp);
				}
/**** sample: execute "sample_action.sh" if it's exist: end ****/

				cnt_char = 0;
				analen = 0;
			}
		}
	}
/**** sample: Write data and Read response: end ****/

	return EXIT_SUCCESS;
}


